@@ -88,6 +88,77 @@ northwestcmp (gconstpointer a, gconstpointer b)
     return 0;
 }
 
+static gboolean
+place_by_pointer(MetaWindow *window,
+                 MetaFrameBorders *borders,
+                 MetaPlacementMode placement_mode,
+                 int *new_x,
+                 int *new_y)
+{
+  int window_width, window_height;
+  Window root_return, child_return;
+  int root_x_return, root_y_return;
+  int win_x_return, win_y_return;
+  unsigned int mask_return;
+
+  XQueryPointer (window->display->xdisplay,
+                 window->screen->xroot,
+                 &root_return,
+                 &child_return,
+                 &root_x_return,
+                 &root_y_return,
+                 &win_x_return,
+                 &win_y_return,
+                 &mask_return);
+
+  window_width = window->frame ? window->frame->rect.width : window->rect.width;
+  window_height = window->frame ? window->frame->rect.height : window->rect.height;
+
+  if (borders) {
+    *new_x = root_x_return + borders->visible.left - window_width / 2;
+    *new_y = root_y_return + borders->visible.top  - window_height / 2;
+  }
+  else {
+    *new_x = root_x_return - window_width / 2;
+    *new_y = root_y_return - window_height / 2;
+  }
+
+  if (placement_mode == META_PLACEMENT_MODE_MANUAL)
+    window->move_after_placement = TRUE;
+
+  return TRUE;
+}
+
+static gboolean
+place_in_center (MetaWindow *window,
+                 MetaFrameBorders *borders,
+                 MetaPlacementMode placement_mode,
+                 int *new_x,
+                 int *new_y)
+{
+  int center_x, center_y;
+  const MetaMonitorInfo *xi;
+
+  xi = meta_screen_get_current_monitor (window->screen);
+  center_x = *new_x + xi->rect.width / 2;
+  center_y = *new_y + xi->rect.height / 2;
+
+  int window_width, window_height;
+  window_width = window->frame ? window->frame->rect.width : window->rect.width;
+  window_height = window->frame ? window->frame->rect.height : window->rect.height;
+
+  if (borders) {
+    *new_x = center_x + borders->visible.left - window_width / 2;
+    *new_y = center_y + borders->visible.top  - window_height / 2;
+  }
+  else {
+    *new_x = center_x - window_width / 2;
+    *new_y = center_y - window_height / 2;
+  }
+
+  return TRUE;
+}
+
 static void
 find_next_cascade (MetaWindow *window,
                    MetaFrameBorders *borders,
@@ -101,6 +172,7 @@ find_next_cascade (MetaWindow *window,
   GList *tmp;
   GList *sorted;
   int cascade_x, cascade_y;
+  MetaRectangle titlebar_rect;
   int x_threshold, y_threshold;
   int window_width, window_height;
   int cascade_stage;
@@ -120,17 +192,11 @@ find_next_cascade (MetaWindow *window,
    * manually cascade.
    */
 #define CASCADE_FUZZ 15
-  if (borders)
-    {
-      x_threshold = MAX (borders->visible.left, CASCADE_FUZZ);
-      y_threshold = MAX (borders->visible.top, CASCADE_FUZZ);
-    }
-  else
-    {
-      x_threshold = CASCADE_FUZZ;
-      y_threshold = CASCADE_FUZZ;
-    }
-  
+
+  meta_window_get_titlebar_rect (window, &titlebar_rect);
+  x_threshold = MAX (titlebar_rect.x, CASCADE_FUZZ);
+  y_threshold = MAX (titlebar_rect.y, CASCADE_FUZZ);
+
   /* Find furthest-SE origin of all workspaces.
    * cascade_x, cascade_y are the target position
    * of NW corner of window frame.
@@ -175,9 +241,10 @@ find_next_cascade (MetaWindow *window,
            * point. The new window frame should go at the origin
            * of the client window we're stacking above.
            */
-          meta_window_get_position (w, &wx, &wy);
-          cascade_x = wx;
-          cascade_y = wy;
+          meta_window_get_titlebar_rect (w, &titlebar_rect);
+          /* Cascade the window evenly by the titlebar height; this isn't a typo. */
+          cascade_x = wx + titlebar_rect.height;
+          cascade_y = wy + titlebar_rect.height;
           
           /* If we go off the screen, start over with a new cascade */
 	  if (((cascade_x + window_width) >
@@ -660,6 +727,8 @@ meta_window_place (MetaWindow        *window,
 {
   GList *windows;
   const MetaMonitorInfo *xi;
+  MetaPlacementMode placement_mode;
+  MetaWindow *parent;
 
   /* frame member variables should NEVER be used in here, only
    * MetaFrameBorders. But remember borders == NULL
@@ -671,7 +740,9 @@ meta_window_place (MetaWindow        *window,
   meta_topic (META_DEBUG_PLACEMENT, "Placing window %s\n", window->desc);
 
   windows = NULL;
-  
+  parent = meta_display_lookup_x_window (window->display,
+                                         window->xtransient_for);
+
   switch (window->type)
     {
       /* Run placement algorithm on these. */
@@ -700,7 +771,8 @@ meta_window_place (MetaWindow        *window,
       goto done_no_constraints;
     }
   
-  if (meta_prefs_get_disable_workarounds ())
+  if (meta_prefs_get_disable_workarounds () ||
+      (parent != NULL && parent->type == META_WINDOW_DESKTOP))
     {
       switch (window->type)
         {
@@ -763,20 +835,14 @@ meta_window_place (MetaWindow        *window,
   
   if ((window->type == META_WINDOW_DIALOG ||
        window->type == META_WINDOW_MODAL_DIALOG) &&
-      window->xtransient_for != None)
+      window->xtransient_for != None &&
+      (parent != NULL && parent->type != META_WINDOW_DESKTOP))
     {
       /* Center horizontally, at top of parent vertically */
 
-      MetaWindow *parent;
-          
-      parent =
-        meta_display_lookup_x_window (window->display,
-                                      window->xtransient_for);
-
       if (parent)
         {
           int w;
-
           meta_window_get_position (parent, &x, &y);
           w = parent->rect.width;
 
@@ -825,7 +891,9 @@ meta_window_place (MetaWindow        *window,
 
       x += xi->rect.x;
       y += xi->rect.y;
-      
+
+      avoid_being_obscured_as_second_modal_dialog (window, borders, &x, &y);
+
       meta_topic (META_DEBUG_PLACEMENT, "Centered window %s on screen %d monitor %d\n",
                   window->desc, window->screen->number, xi->number);
 
@@ -866,6 +934,23 @@ meta_window_place (MetaWindow        *window,
   x = xi->rect.x;
   y = xi->rect.y;
 
+  
+  /* Placement based on pointer position */
+  placement_mode = meta_prefs_get_placement_mode();
+
+  if (placement_mode == META_PLACEMENT_MODE_POINTER ||
+      placement_mode == META_PLACEMENT_MODE_MANUAL)
+    {
+      if (place_by_pointer (window, borders, placement_mode, &x, &y))
+        goto done_check_denied_focus;
+    }
+
+  else if (placement_mode == META_PLACEMENT_MODE_CENTER)
+    {
+      if (place_in_center (window, borders, placement_mode, &x, &y))
+        goto done_check_denied_focus;
+    }
+
   if (find_first_fit (window, borders, windows,
                       xi->number,
                       x, y, &x, &y))
