@@ -44,7 +44,6 @@
 #include <X11/Xutil.h>
 #include <cairo.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include "tile-hud.h"
 
 typedef struct _MetaWindowQueue MetaWindowQueue;
 
@@ -94,6 +93,7 @@ enum {
 #define NUMBER_OF_QUEUES 3
 
 #define HUD_WIDTH 24
+#define CSD_TITLEBAR_HEIGHT 48
 
 struct _MetaWindow
 {
@@ -548,7 +548,7 @@ void        meta_window_unmanage           (MetaWindow  *window,
 void        meta_window_calc_showing       (MetaWindow  *window);
 void        meta_window_queue              (MetaWindow  *window,
                                             guint queuebits);
-void        meta_window_tile               (MetaWindow        *window,
+void        meta_window_real_tile          (MetaWindow        *window,
                                             gboolean           force);
 void        meta_window_maximize_internal  (MetaWindow        *window,
                                             MetaMaximizeFlags  directions,
@@ -702,6 +702,8 @@ void meta_window_get_work_area_for_monitor     (MetaWindow    *window,
 void meta_window_get_work_area_all_monitors    (MetaWindow    *window,
                                                 MetaRectangle *area);
 
+int meta_window_get_current_tile_monitor_number (MetaWindow *window);
+
 void meta_window_get_current_tile_area         (MetaWindow    *window,
                                                 MetaRectangle *tile_area);
 
@@ -711,6 +713,9 @@ void meta_window_get_tile_threshold_area_for_mode        (MetaWindow    *window,
                                                           MetaRectangle *tile_area,
                                                           gint           zone_width);
 
+void meta_window_get_titlebar_rect (MetaWindow *window,
+                                    MetaRectangle *titlebar_rect);
+
 gboolean meta_window_same_application (MetaWindow *window,
                                        MetaWindow *other_window);
 
@@ -765,16 +770,8 @@ gboolean meta_window_can_tile_top_bottom (MetaWindow *window);
 gboolean meta_window_can_tile_corner         (MetaWindow *window);
 MetaSide meta_window_get_tile_side (MetaWindow *window);
 
-inline void meta_window_get_size_limits   (const MetaWindow        *window,
-                                          const MetaFrameBorders  *borders,
-                                          gboolean include_frame,
-                                          MetaRectangle *min_size,
-                                          MetaRectangle *max_size);
-
 void meta_window_compute_tile_match (MetaWindow *window);
 
-HUDTileRestrictions meta_window_get_tile_restrictions (MetaWindow *window);
-
 gboolean meta_window_mouse_on_edge (MetaWindow *window,
                                     gint        x,
                                     gint        y);
